{-# LANGUAGE OverloadedStrings #-}

module Main where

import Conduit ( sinkList, mapM_C, (.|), runConduit )
import Data.LanguageCodes (ISO639_1(..))
import Data.Text qualified as T
import HAL (getMetadataWithCursorOptsC, countResultsOpts', HalCrawlerOptions(..), defaultHalOptions, getMetadataWith, generateRequestByStructID)
import HAL.Types
import HAL.Doc
import HAL.Doc.Corpus (Corpus(..))
import Network.HTTP.Client (newManager)
import Network.HTTP.Client.TLS (tlsManagerSettings)
import Options.Applicative
import Prelude qualified
import Protolude


data CountParams = CountParams
  { cp_query :: T.Text
  , cp_lang  :: Maybe ISO639_1 }

data FetchParams = FetchParams
  { fp_query :: T.Text
  , fp_limit :: Integer
  , fp_lang  :: Maybe ISO639_1 }

data Command =
    Count CountParams
  | Fetch FetchParams

countParams :: Parser Command
countParams = Count <$>
  (CountParams
  <$> strArgument (metavar "query")
  <*> optional (option (maybeReader readLang) (long "lang")))

fetchParams :: Parser Command
fetchParams = Fetch <$>
  (FetchParams
  <$> strArgument (metavar "query")
  <*> option auto (long "limit")
  <*> optional (option (maybeReader readLang) (long "lang")))

readLang :: Prelude.String -> Maybe ISO639_1
readLang = readMaybe

params :: Parser Command
params = subparser
  (command "count" (info countParams (progDesc "Count number of docs for a given query"))
  <> command "fetch" (info fetchParams (progDesc "Fetch docs for a given query")))

opts :: ParserInfo Command
opts = info (params <**> helper)
       (fullDesc
       <> progDesc "A program to test HAL"
       <> header "crawlerHAL-exe")

main :: IO ()
main = do run =<< execParser opts
  -- res <- getMetadataWith [generateRequestByStructID "(en_title_t:(glass) OR en_abstract_t:(glass)) AND (language_s:en)" imt] (Just 0) (Just 555) (Just EN)

  -- case res of
  --   (Left  err) -> print err
  --   (Right val) -> do
  --     mapM_ printCorpus $ _docs val

-- this function is for debug purpose
printCorpus :: MonadIO m => Corpus -> m ()
printCorpus Corpus { .. } = do
  putText $ "StructId: [" <> T.intercalate ", "  (map (T.pack . show) _corpus_struct_id)  <> "]"
  putText $ "Authors affiliations: [" <> T.intercalate ", " _corpus_authors_affiliations <> "]"
  putText "------------"


run :: Command -> IO ()
run (Count (CountParams { cp_query, cp_lang })) = do
  res <- countResultsOpts' opts cp_query cp_lang
  case res of
    Left err -> putText $ show err
    Right cnt -> putText $ show cnt
  where
    opts = defaultHalOptions { _hco_debugLogs = True }
run (Fetch (FetchParams { fp_query, fp_limit, fp_lang })) = do
  res <- getMetadataWithCursorOptsC opts fp_query (Just fp_limit) fp_lang
  case res of
    Left err -> putText $ show err
    Right (_cnt, docsC) -> do
      _ <- runConduit $
           docsC
        .| mapM_C printCorpus
        .| sinkList
      pure ()
  where
    opts = defaultHalOptions { _hco_debugLogs = True }
    printCorpus Corpus { .. } = do
      putText $ "docid: " <> _corpus_docid <> " [" <> (T.intercalate " " _corpus_title) <> "]"
      putText $ "  " <> (T.intercalate " " _corpus_abstract)
      putText $ "  " <> show _corpus_abstract_lang_map
      putText $ "  " <> show _corpus_original
      putText "------------"


-- data
-- yearReq = [text|
-- (language_t:en)
-- AND (producedDateY_i:2018
--      OR producedDateY_i:2019
--      OR producedDateY_i:2020
--      OR producedDateY_i:2021
--      OR producedDateY_i:2022)
-- AND (structId_i:302102
--     OR structId_i:469216
--     OR structId_i:6279
--     OR structId_i:224096
--     OR structId_i:144103
--     OR structId_i:497330
--     OR structId_i:1076752
--     OR structId_i:84538
--     OR structId_i:301262
--     OR structId_i:481355
--     OR structId_i:29212
--     OR structId_i:301442
--     OR structId_i:542824
--     OR structId_i:300362
--     OR structId_i:1048346
--     OR structId_i:352124)|]

-- imt :: [T.Text]
-- imt = [
--   "224096"
--   ,"144103"
--   ,"84538"
--   ,"300104"
--   ,"300362"
--   ,"301262"
--   ,"301442"
--   ,"301492"
--   ,"302102"
--   ,"421532"
--   ,"497330"
--   ,"352124"
--   ,"481355"
--   ,"469216"
--   ,"542824"
--   ,"6279"
--   ,"29212"
--  ]
