{-# LANGUAGE NoMonomorphismRestriction#-}

module HAL.Client where

import Data.Aeson (FromJSON)
import HAL.Types (Response)
import Protolude
import Servant.API
import Servant.Client (ClientM, client)


type HALAPI doc = Search doc
  :<|> SearchCursor doc
  :<|> Structure doc

type Search doc = "search"
  -- q is the main query
  :> QueryParam "q" Text
  -- fl determine which fields will be returned it can be a list of fields or *
  :> QueryParam "fl" Text -- doc
  -- TODO: type this monster
  -- fq is to filter request
  :> QueryParams "fq" Text
  -- pretty much clear, (Asc || Desc) + field you want to sort by
  :> QueryParam "sort" SortField
  -- permit to start at the x result
  :> QueryParam "start" Int
  -- use rows to make the request only return the x number of result
  :> QueryParam "rows" Int
  :> Get '[JSON] (Response doc)

type SearchCursor doc = "search"
  -- q is the main query
  :> QueryParam "q" Text
  -- fl determine which fields will be returned it can be a list of fields or *
  :> QueryParam "fl" Text -- doc
  -- TODO: type this monster
  -- fq is to filter request
  :> QueryParams "fq" Text
  -- pretty much clear, (Asc || Desc) + field you want to sort by
  -- is required when using cursor
  :> QueryParam "sort" SortField
  -- permit to start at the x result
  :> QueryParam "rows" Int
  -- cursor
  :> QueryParam "cursorMark" Text
  :> Get '[JSON] (Response doc)

type Structure doc = "ref" :> "structure"
  :> QueryParam "fl" Text
  :> QueryParam "fq" Text
  :> QueryParam "rows" Int
  :> Get '[JSON] (Response doc)

-- Get's argument type
data SortField = Asc Text | Desc Text
  deriving (Show)

instance ToHttpApiData SortField where
  toUrlPiece (Asc  t) = t <> " asc"
  toUrlPiece (Desc t) = t <> " desc"

asc :: Text -> Maybe SortField
asc = Just . Asc

desc :: Text -> Maybe SortField
desc = Just . Desc

halAPI :: Proxy (HALAPI doc)
halAPI = Proxy

-- search should always have at least `docid` and `label_s` in his fl params
search :: (FromJSON doc, ToHttpApiData doc) =>
     Maybe Text  -- q
  -> Maybe Text -- fl
  -> [Text] -- fq
  -> Maybe SortField -- sort
  -> Maybe Int -- start
  -> Maybe Int -- rows
  -> ClientM (Response doc)

-- search should always have at least `docid` and `label_s` in his fl params
searchCursor :: (FromJSON doc, ToHttpApiData doc) =>
     Maybe Text  -- q
  -> Maybe Text -- fl
  -> [Text] -- fq
  -> Maybe SortField -- sort
  -> Maybe Int -- rows
  -> Maybe Text -- cursor
  -> ClientM (Response doc)

structure :: (FromJSON doc, ToHttpApiData doc) =>
  Maybe Text
  -> Maybe Text -- fq
  -> Maybe Int -- rows
  -> ClientM (Response doc)

(search :<|> searchCursor :<|> structure) = client halAPI
